// ETUDriverSink.h : Declaration of the CETUDriverSink

#pragma once
#include "resource.h"       // main symbols
#include "ETUDSink_i.h"

#include "IGazeProcessor.h"
#include "GazeCallbacks.h"

#import "C:\\Program Files (x86)\\University of Tampere\\ETUDriver\\ETUDriver.dll" no_namespace

#if defined(_WIN32_WCE) && !defined(_CE_DCOM) && !defined(_CE_ALLOW_SINGLE_THREADED_OBJECTS_IN_MTA)
#error "Single-threaded COM objects are not properly supported on Windows CE platform, such as the Windows Mobile platforms that do not include full DCOM support. Define _CE_ALLOW_SINGLE_THREADED_OBJECTS_IN_MTA to force ATL to support creating single-thread COM object's and allow use of it's single-threaded COM object implementations. The threading model in your rgs file was set to 'Free' as that is the only threading model supported in non DCOM Windows CE platforms."
#endif

using namespace ATL;

typedef boost::shared_ptr<IGazeProcessor> IGazeProcessorPtr;

extern _ATL_FUNC_INFO OnDataEventInfo;

// CETUDriverSink

class /*ATL_NO_VTABLE*/ CETUDriverSink :
	public CComObjectRootEx<CComSingleThreadModel>,
	public CComCoClass<CETUDriverSink, &CLSID_ETUDriverSink>,
	public IDispatchImpl<IETUDriverSink, &IID_IETUDriverSink, &LIBID_ETUDSinkLib, /*wMajor =*/ 1, /*wMinor =*/ 0>,
	public IDispEventSimpleImpl<1, CETUDriverSink, &__uuidof(IETUDriverEvents)>
{
public:
	CETUDriverSink()
	{
		mSampleClbk = NULL;
		mBlinkClbk = NULL;
		mFixationClbk = NULL;
	}

DECLARE_REGISTRY_RESOURCEID(IDR_ETUDRIVERSINK)


BEGIN_COM_MAP(CETUDriverSink)
	COM_INTERFACE_ENTRY(IETUDriverSink)
	COM_INTERFACE_ENTRY(IDispatch)
END_COM_MAP()


	DECLARE_PROTECT_FINAL_CONSTRUCT()

	HRESULT FinalConstruct()
	{
		return S_OK;
	}

	void FinalRelease()
	{
	}

	void __stdcall OnDataEvent(long dataEvent, long* data, long* param);

	BEGIN_SINK_MAP(CETUDriverSink)
		SINK_ENTRY_INFO(/*nID =*/ 1, __uuidof(IETUDriverEvents), /*dispid =*/ 5, OnDataEvent, &OnDataEventInfo)
	END_SINK_MAP()

public:
	/*void __stdcall SetFixationCallback(fixationCallback clbk);
	void __stdcall SetSampleCallback(sampleCallback clbk);
	void __stdcall SetBlinkCallback(blinkCallback clbk);*/

	void __stdcall SetGazeProcessor(IGazeProcessorPtr processor){
		pProcessor = processor; 
	}

	STDMETHOD(Start)()
	{
		if (pETUD)
			return S_OK;

		HRESULT hr = CoCreateInstance(__uuidof(CoETUDriver), NULL, CLSCTX_INPROC_SERVER, __uuidof(IETUDriver), (LPVOID*)&pETUD);

	  if (FAILED(hr)){
		  return hr;
	  }

	  ATLASSERT(pETUD != NULL);

	  pETUD->OptionsFile = "options.ini";

	  if(pETUD->Index < 0)
	  {
		wchar_t* buff = new wchar_t[128];
		for(int i = 0; i < pETUD->DeviceCount; i++)
		{
			pETUD->getNameFromIndex(i, &buff, NULL);
			if(wcscmp(buff, L"Mouse") == 0)
			{
				pETUD->Index = i;
				break;
			}
		}
		delete buff;
	  }
	  DispEventAdvise(pETUD);
      return S_OK;
   }

   STDMETHOD(Stop)()
   {
      if (!pETUD)
         return S_OK;

      DispEventUnadvise(pETUD);

      pETUD.Release();

	  return S_OK;
   }

   STDMETHOD(IsTracking)()
   {
      if (!pETUD){
		return S_FALSE;
	  }

	  return pETUD->GetActive() ? S_OK : S_FALSE;
   }

   STDMETHOD(Options)()
   {
      if (!pETUD){
         return S_FALSE;
	  }

	  pETUD->showRecordingOptions();

	  return S_OK;
   }

   STDMETHOD(Source)()
   {
      if (!pETUD)
         return S_FALSE;

	  pETUD->showDeviceOptions();

	  return S_OK;
   }

   STDMETHOD(Calibrate)()
   {
      if (!pETUD)
         return S_FALSE;

	  pETUD->calibrate();

	  return S_OK;
   }

   STDMETHOD(StartTracking)()
   {
      if (!pETUD)
         return S_FALSE;

      if (pETUD->Active)
         return S_FALSE;

	  pETUD->startTracking();
	  //MessageBox(NULL, TEXT("1"), TEXT("StartTracking"), MB_OK);
	  return S_OK;
   }

   STDMETHOD(StopTracking)()
   {
      if (!pETUD)
         return S_FALSE;

      if (!pETUD->Active)
         return S_FALSE;

	  pETUD->stopTracking();

	  return S_OK;
   }

private:
	IETUDriverPtr pETUD;
	IGazeProcessorPtr pProcessor;

	sampleCallback mSampleClbk;
	blinkCallback mBlinkClbk;
	fixationCallback mFixationClbk;
};

OBJECT_ENTRY_AUTO(__uuidof(ETUDriverSink), CETUDriverSink)
